//sepcial enhancement for Futaba FF6 only
#include "Arduino.h"
#include <avr/interrupt.h>

typedef enum {
 	NOT_SYNCHED,READY
}
State_t;

#define TICKS_PER_uS      1
#define MAX_CHANNELS      8
#define MIN_IN_PULSE  (700 * TICKS_PER_uS)
#define MAX_IN_PULSE  (2200 * TICKS_PER_uS)
#define SYNC_GAP_LEN  (3000 * TICKS_PER_uS)

#define DSM2_CHANNELS      6
#define BINDING_PIN        4
#define GREEN_LED 13
#define SLOW              1000*2
#define FAST              125*2

static int Pulses[  MAX_CHANNELS + 1];
static byte ChannelNum;
static State_t State;

static byte DSM2_Channel[2+DSM2_CHANNELS*2] = {0x80,0x00,0x00,0xAA,0x05,0xFF,0x09,0xFF,0x0D,0xFF,0x13,0x54,0x14,0xAA};
static byte ChanIndex[] = {3,1,2,4,5,6};
static byte count;
static int pulse;
static int  flash_duration ;
static unsigned long startTime = 0;

ISR(TIMER1_OVF_vect) {
  if(State == READY) {
    State = NOT_SYNCHED;
    ChannelNum = 0;
  }
}

ISR(TIMER1_CAPT_vect) {
  TCNT1 = 0;
  if(ICR1 >= SYNC_GAP_LEN){
    ChannelNum = 0;
	State = READY;
    Pulses[0] = ICR1 / TICKS_PER_uS;
  }
  else
    if((ICR1 >= MIN_IN_PULSE) && (ICR1 <= MAX_IN_PULSE) && State == READY){
      Pulses[++ChannelNum] = ICR1 / TICKS_PER_uS;
    }
}

class PPM_Decode {

public:
  void begin() {
    pinMode(8, INPUT);
    digitalWrite(8, HIGH);
    ChannelNum = 0;
    State   = NOT_SYNCHED;
    TCCR1A  = 0x00;
    TCCR1B = (1<<CS11);
    TIMSK1 = _BV(ICIE1)|_BV (TOIE1);
    for(byte ch = 1; ch <= MAX_CHANNELS; ch++) {
      Pulses[ch] = 1520;
    }
    Pulses[3] = 1100;
  }

  State_t getState() {
    return State;
  }

int getChannelData(uint8_t channel) {     // this is the access function for channel data
    int result;
    cli();
    result = Pulses[channel];
    sei();
    return result;
  }


};

PPM_Decode Receiver = PPM_Decode();

void setup() {
  delay(100);
  Serial.begin(125000);
  Receiver.begin();
  pinMode(BINDING_PIN, INPUT);
  digitalWrite(BINDING_PIN, HIGH);
  pinMode(GREEN_LED,   OUTPUT);
  digitalWrite(GREEN_LED, LOW);
  count = 50;
  while(Receiver.getState() != READY && count-- > 0)
    delay(100);
  while(digitalRead(BINDING_PIN) == LOW) {
    digitalWrite(GREEN_LED, HIGH);
   Serial.write(DSM2_Channel,2+ DSM2_CHANNELS*2);
    delay(20);
  }
  digitalWrite(GREEN_LED,LOW);
  DSM2_Channel[0] = 0x00;
  delay(100);
}

void loop() {
  if(Receiver.getState() == READY) {
    flash_duration = SLOW ;
  }
  else {
    flash_duration = FAST ;
  }
  if(millis()%(flash_duration) < flash_duration/2)
    digitalWrite(GREEN_LED, LOW);
  else
    digitalWrite(GREEN_LED, HIGH);
  if(Receiver.getState() == READY) {
    if(ChannelNum == MAX_CHANNELS) {
      for (byte i=0; i<DSM2_CHANNELS; i++) {
        pulse = Receiver.getChannelData(ChanIndex[i]) - 1000;
        pulse = constrain(pulse, 0, 0x3FF);
        DSM2_Channel[2+i*2]   = (byte)(i<<2) | highByte(pulse);
        DSM2_Channel[2+i*2+1] = lowByte(pulse);
      }
    }
    if (millis()-startTime > 19){
      startTime = millis();
      Serial.write(DSM2_Channel,2+ DSM2_CHANNELS*2);
    }
  }
}

